# vim:fileencoding=utf-8

import re

from django.http import HttpResponse, HttpResponseRedirect, HttpResponsePermanentRedirect, HttpResponseGone
from django.conf import settings
from django.template.loader import render_to_string
from django.utils.http import urlquote

__all__ = (
    'redirect_to',
    'http_response',
    'http_404',
    'http_500',
)

RE_QUOTE = re.compile(r"%([0-9A-Z]{1,2})")

def redirect_to(request, url, permanent=True, **kwargs):
    """
    A copy of the redirect_to view from django.views.generic.simple
    Copied since redirect_to doesn't work for urls that contain non-ascii
    keyword arguments.
    """
    if url is not None:
        klass = permanent and HttpResponsePermanentRedirect or HttpResponseRedirect
        quoted_kwargs = {}
        for k,v in kwargs.items():
            quoted_kwargs[k] = urlquote(v)

        # Encoded urls confuses python templating. Properly escape the templates.
        return klass(urlquote(RE_QUOTE.sub(r"%%\1", url) % quoted_kwargs))
    else:
        return HttpResponseGone()

def http_response(request, template, status=200):
    return HttpResponse(render_to_string(template, {'MEDIA_URL': settings.MEDIA_URL}), status=status)

def http_404(request, template='404.html'):
    return http_response(request, template, status=404)

def http_500(request, template='500.html'):
    return http_response(request, template, status=500)



#:coding=utf-8:

import re
from six import text_type

try:
    import json
except ImportError:
    import simplejson as json

from django.forms import RegexField
from django.utils.translation import ugettext_lazy as _

__all__ = (
    'StripRegexField',
    'EmailField',
    'AlphaNumField',
    'NumCharField',
    'FullWidthCharField',
    'HiraganaCharField',
)

RE_EMAIL = re.compile(
    r"^[-\.!#$%&'*+/=?^_`{}|~0-9A-Z]+"  # account
    r"@(?:[A-Z0-9]+(?:-*[A-Z0-9]+)*\.)+(xn--[A-Z0-9\-]+|[A-Z]{2,15})$",  # domain
    re.IGNORECASE
)
RE_ALPHA_NUM = re.compile(r'^[a-zA-Z0-9\-_]*$')
RE_NUM = re.compile(r'^[0-9]*$')
RE_FULL_WIDTH = re.compile(u'[一-龠]+|[ぁ-ん]+|[ァ-ヴ]+|[０-９]+')
RE_HIRAGANA = re.compile(u'^[ぁ-ゞー〜～＆ 　、・]*$')


class StripRegexField(RegexField):
    """
    検証する前にstripする正規表現のフィールド
    """
    def clean(self, value):
        if isinstance(value, text_type):
            value = value.strip()
        return super(StripRegexField, self).clean(value)


class EmailField(StripRegexField):
    default_error_messages = {
        'invalid': _(u'Eメールアドレスの形式が不正です'),
    }

    def __init__(self, *args, **kwargs):
        super(EmailField, self).__init__(RE_EMAIL, *args, **kwargs)


class AlphaNumField(StripRegexField):
    """
    半角英数字と"_","-"のみ許容するフィールド
    """
    default_error_messages = {
        'invalid': _(u'半角英数字で入力してください'),
    }

    def __init__(self, *args, **kwargs):
        super(AlphaNumField, self).__init__(RE_ALPHA_NUM, *args, **kwargs)


class NumCharField(StripRegexField):
    """
    数字のみを許容するフィールド
    """
    default_error_messages = {
        'invalid': _(u'数字で入力してください'),
    }

    def __init__(self, *args, **kwargs):
        super(NumCharField, self).__init__(RE_NUM, *args, **kwargs)


class FullWidthCharField(StripRegexField):
    """
    全角文字のみを許容するフィールド
    """
    default_error_messages = {
        'invalid': _(u'全角文字を入力してください。'),
    }

    def __init__(self, *args, **kwargs):
        super(FullWidthCharField, self).__init__(RE_FULL_WIDTH, *args, **kwargs)


class HiraganaCharField(StripRegexField):
    u"""
    全角ひらがなのみ許容するフィールド
    """
    default_error_messages = {
        'invalid': _(u'ひらがなで入力してください'),
    }

    def __init__(self, *args, **kwargs):
        super(HiraganaCharField, self).__init__(RE_HIRAGANA, *args, **kwargs)



# vim:fileencoding=utf-8

from django.template import Node, Library, TemplateDoesNotExist, TemplateSyntaxError, Variable
from django.template.loader import render_to_string

register = Library()

class GoogleAdNode(Node):
    def __init__(self, template_name_var, block_id_var=None):
        self.block_id_var = Variable(block_id_var) if block_id_var else None
        self.template_name_var = Variable(template_name_var)

    def render(self, context):
        try:
            block_id = self.block_id_var.resolve(context) if self.block_id_var else None;
            template_name = self.template_name_var.resolve(context)
            context_dict = {
                "block_id": block_id if block_id else "",
            }
            for c in context.dicts:
                context_dict.update(c)

            return render_to_string(template_name, context_dict)

        except TemplateDoesNotExist as e:
            return ''

@register.tag
def googlead(parser, token):
    bits = token.split_contents()
    if len(bits) != 2 and len(bits) != 3:
        raise TemplateSyntaxError("%r takes at least one argument." % bits[0])
    template_name = bits[1]
    if len(bits) == 3:
        return GoogleAdNode(template_name, bits[2])
    else:
        return GoogleAdNode(template_name)



import string
import re
import json

import six
from django.utils.encoding import force_text
from django.core.serializers.json import DjangoJSONEncoder
try:
    from django.utils.functional import keep_lazy  # Django-1.8 doesn't have it.
except ImportError:
    # allow_lazy has been deprecated at Django-1.10, will be removed at 2.0
    from django.utils.functional import allow_lazy as keep_lazy
    

# copy from beproud.utils.strutils.abbrev
def abbrev(s, num=255, end="..."):
    """
    文章を要約する
    返す文字列の長さは、num以上にならないのを保証します。

    >>> abbrev('spamspamspam', 6)
    'spa...'
    >>> abbrev('spamspamspam', 12)
    'spamspamspam'
    >>> abbrev('eggseggseggs', 1)
    'e'
    >>> abbrev('eggseggseggs', 2)
    'eg'
    >>> abbrev('eggseggseggs', 3)
    'egg'
    >>> abbrev('eggseggseggs', 4)
    'e...'
    >>> abbrev('eggseggseggs', 2, '.')
    'e.'
    """
    index = num - len(end)
    if len(s) > num:
        s = (s[:index] + end) if index > 0 else s[:num]
    return s


# copy from beproud.utils.html.urlize
def escape(html):
    """
    Returns the given HTML with ampersands, quotes and angle brackets encoded.
    """
    return (force_text(html).replace('&', '&amp;')
            .replace('<', '&lt;')
            .replace('>', '&gt;')
            .replace('"', '&quot;')
            .replace("'", '&#39;'))


# copy from beproud.utils.html.urlize
HTTP_SCHEME_RE = 'http[s]*'

# See: http://www.ietf.org/rfc/rfc1738.txt
URL_SAFE = "$-_.+"
URL_EXTRA = "!*'(),"
URL_PATH_RESERVED = ';?'
URL_QUERY_RESERVED = '#'
URL_OTHER_RESERVED = ':@&=/'
URL_RESERVED = URL_PATH_RESERVED + URL_QUERY_RESERVED + URL_OTHER_RESERVED
URL_ESCAPE = '%'
URL_ALNUM = string.ascii_letters + string.digits


URL_PATH_VALID_CHARS = URL_ALNUM + URL_SAFE + URL_EXTRA + URL_OTHER_RESERVED + URL_ESCAPE
URL_QUERY_VALID_CHARS = URL_ALNUM + URL_SAFE + URL_EXTRA + URL_OTHER_RESERVED + URL_PATH_RESERVED + URL_ESCAPE
URL_FRAGMENT_VALID_CHARS = URL_ALNUM + URL_SAFE + URL_EXTRA + URL_RESERVED + URL_ESCAPE


# 0-65535
# See: http://www.regular-expressions.info/numericranges.html
PORT_RE = "%s" % "|".join([
    "6553[0-5]",
    "655[0-2][0^9]",
    "65[0-4][0-9][0-9]",
    "6[0-4][0-9][0-9][0-9]",
    "[1-5][0-9][0-9][0-9][0-9]",
    "[1-9][0-9][0-9][0-9]",
    "[1-9][0-9][0-9]",
    "[1-9][0-9]",
    "[1-9]",
])

# See: http://www.shauninman.com/archive/2006/05/08/validating_domain_names
# See: http://www.iana.org/domains/root/db/
DOMAIN_RE = '(?:[a-z0-9](?:[-a-z0-9]*[a-z0-9])?\\.)+(?:(?:aero|arpa|a[cdefgilmnoqrstuwxz])|(?:cat|com|coop|b[abdefghijmnorstvwyz]|biz)|(?:c[acdfghiklmnorsuvxyz])|d[ejkmoz]|(?:edu|e[ceghrstu])|f[ijkmor]|(?:gov|g[abdefghilmnpqrstuwy])|h[kmnrtu]|(?:info|int|i[delmnoqrst])|(?:jobs|j[emop])|k[eghimnprwyz]|l[abcikrstuvy]|(?:mil|mobi|museum|m[acdghklmnopqrstuvwxyz])|(?:name|net|n[acefgilopruz])|(?:om|org)|(?:pro|p[aefghklmnrstwy])|qa|r[eouw]|s[abcdeghijklmnortvyz]|(?:travel|t[cdfghjklmnoprtvwz])|u[agkmsyz]|v[aceginu]|w[fs]|y[etu]|z[amw])'

# See: http://www.regular-expressions.info/regexbuddy/ipaccurate.html
IP_ADDRESS_RE = '(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)'

# Domain or IP address
IP_DOMAIN_RE = '(%s)|(%s)' % (DOMAIN_RE, IP_ADDRESS_RE)

# Domain or IP address with port number
URL_DOMAIN_RE = '(?:%s)(?::(%s))?' % (IP_DOMAIN_RE, PORT_RE)
URL_RE = r'(%s)\:\/\/(%s)(/[%s]*)?(?:\?([%s]*))?(?:\#([%s]*))?' % (
    HTTP_SCHEME_RE,
    URL_DOMAIN_RE,
    re.escape(URL_PATH_VALID_CHARS),
    re.escape(URL_QUERY_VALID_CHARS),
    re.escape(URL_FRAGMENT_VALID_CHARS),
)
URL_RE_CMP = re.compile(URL_RE)

URLIZE_TMPL = '<a href="%(link_url)s"%(attrs)s>%(link_text)s</a>'


# copy from beproud.utils.html.urlize
def urlize(text, trim_url_limit=None, attrs={}, url_re=URL_RE_CMP, autoescape=False):
    """text内URLを抽出してアンカータグで囲む

    URLのデリミタは半角カンマ、<>(エスケープ済み含む)、\s、全角スペース、行末で、これらが末尾にマッチしない場合はURLとして認識しません。
    URL部分は.+の最小マッチ、もしくはtrim_url_limitが指定された場合は{,trim_url_limit}の最小マッチとなります。

    -args

        text:           urlize対象文字列
        trim_url_limit: urlとして認識する文字数に上限を設ける場合は数値をセット
        nofollow:       Trueを与えるとタグにrel="nofollow"を付加
        autoescape:     Trueを与えるとタグエスケープを行います。

    """
    if autoescape:
        text = escape(text)

    def _repl(m):
        return URLIZE_TMPL % {
            "link_url": m.group(),
            "attrs": "".join(map(lambda x: ' %s="%s"' % x, attrs.items())),
            "link_text": (abbrev(m.group(), trim_url_limit)
                          if trim_url_limit is not None else m.group()),
        }

    return url_re.sub(_repl, text)


# copy from django.utils.html.strip_entities(). it was removed
def strip_entities(value):
    """Returns the given HTML with all entities (&something;) stripped."""
    return re.sub(r'&(?:\w+|#\d+);', '', force_text(value))
strip_entities = keep_lazy(strip_entities, six.text_type)


# copy from bputils: beproud.utils.javascript
JS_CONVERT_TYPES = {
    'bool': bool,
    'int': int,
    'string': str,
    'array': list,
}


# copy from bputils: beproud.utils.javascript
def force_js(value, typename=None, encoder=None):
    """
    Changes a python value to javascript for use in templates
    """
    if typename:
        typename = typename.lower()
        if typename in JS_CONVERT_TYPES:
            value = JS_CONVERT_TYPES[typename](value)
    return json.dumps(value, cls=(encoder or DjangoJSONEncoder))



# vim:fileencoding=utf-8
from __future__ import division
import time
import calendar
from datetime import datetime, timedelta

SEC_1_MINUTES = 60
SEC_1_HOURS = SEC_1_MINUTES * 60

__all__ = (
    'utc_to_local',
    'relative_time',
    'times_left',
)

def utc_to_local(utc_datetime):
    """
    UTC datetime obj => local datetime obj
    """
    return datetime(*time.localtime(calendar.timegm(utc_datetime.timetuple()))[:6])

def relative_time(d, basetime=None, shortname=False, delta=None):
    """
    経過時間を表す文字列
    """
    if not delta:
        basetime = basetime or datetime.now()
        delta = basetime - d
    if basetime < d or (delta.days < 1 and delta.seconds < 1):
        return u"たった今"
    if delta.days > 0:
        if shortname:
            return d.strftime("%m-%d")
        else:
            return d.strftime("%Y-%m-%d %H:%M")
    elif delta.seconds > SEC_1_HOURS:
        hours = delta.seconds >= SEC_1_MINUTES * 30 and (delta.seconds // SEC_1_HOURS) + 1 or (delta.seconds // SEC_1_HOURS)
        result = u"%s時間前" % (hours)
    elif delta.seconds > SEC_1_MINUTES:
        minutes = delta.seconds >= 30 and (delta.seconds // SEC_1_MINUTES) + 1  or (delta.seconds // SEC_1_MINUTES)
        result = u"%s分前" % (minutes)
    else :
        result = u"%s秒前" % (delta.seconds)
    return result

def times_left(d, basetime=None, delta=None):
    """
    残り時間を表す文字列
    """
    if not delta:
        basetime = basetime or datetime.now()
        delta = d - basetime
    if delta < timedelta(0):
        result = u'終了'
    elif delta.days > 0:
        days = delta.seconds >= SEC_1_HOURS * 12 and delta.days + 1 or delta.days
        result = u"あと%s日" % (days)
    elif delta.seconds > SEC_1_HOURS:
        hours = delta.seconds >= SEC_1_MINUTES * 30 and (delta.seconds // SEC_1_HOURS) + 1 or (delta.seconds // SEC_1_HOURS)
        result = u"あと%s時間" % (hours)
    else:
        minutes = delta.seconds >= 30 and (delta.seconds // SEC_1_MINUTES) + 1  or (delta.seconds // SEC_1_MINUTES)
        result = u"あと%s分" % (minutes)
    return result



# vim:fileencoding=utf-8

import warnings
from django.core.serializers.json import DjangoJSONEncoder  # NOQA

warnings.warn('beproud.django.commons.utils.javascript.DjangoJSONEncoder was deprecated. Use django.core.serializers.json.DjangoJSONEncoder instead.', DeprecationWarning)



#:coding=utf-8:

from functools import wraps

from django.shortcuts import render
from django.http import JsonResponse


def render_to(template=None):
    """
    Decorator for Django views that sends returned dict to render function.

    Template name can be decorator parameter or TEMPLATE item in returned dictionary.
    RequestContext always added as context instance.
    If view doesn't return dict then decorator simply returns output.

    Parameters:
     - template: template name to use

    Examples:
    # 1. Template name in decorator parameters

    @render_to('template.html')
    def foo(request):
        bar = Bar.object.all()  
        return {'bar': bar}

    # equals to 
    def foo(request):
        bar = Bar.object.all()  
        return render(request, 'template.html', {'bar': bar})


    # 2. Template name as TEMPLATE item value in return dictionary

    @render_to()
    def foo(request, category):
        template_name = '%s.html' % category
        return {'bar': bar, 'TEMPLATE': template_name}
    
    #equals to
    def foo(request, category):
        template_name = '%s.html' % category
        return render(request, template_name, {'bar': bar})

    Taken from django-annoying
    """
    def renderer(function):
        @wraps(function)
        def wrapper(request, *args, **kwargs):
            output = function(request, *args, **kwargs)
            if not isinstance(output, dict):
                return output
            tmpl = output.pop('TEMPLATE', template)
            return render(request, tmpl, context=output)
        return wrapper
    return renderer

def ajax_request(func):
    """
    If view returned serializable dict, returns JSONResponse with this dict as content.

    example:
        
        @ajax_request
        def my_view(request):
            news = News.objects.all()
            news_titles = [entry.title for entry in news]
            return {'news_titles': news_titles}

    Taken from django-annoying
    """
    @wraps(func)
    def wrapper(request, *args, **kwargs):
        response = func(request, *args, **kwargs)
        if isinstance(response, (dict, list, tuple)):
            return JsonResponse(response)
        else:
            return response
    return wrapper



#:coding=utf-8:

from __future__ import print_function
from django.conf import settings

class DebugMiddleware(object):
    def process_response(self, request, response):
        if settings.DEBUG: 
            from django.db import connection
            for query in connection.queries:
                print(query)
        return response



#!/usr/bin/env python
#:coding=utf-8:

from setuptools import setup, find_packages
from beproud.django.commons import VERSION


def read(filename):
    with open(filename) as f:
        return f.read()


setup(
    name='beproud.django.commons',
    version=VERSION,
    description='Common utilities for Django',
    long_description=read('README.rst') + read('ChangeLog.rst'),
    author='BeProud Inc.',
    author_email='project@beproud.jp',
    license='BSD',
    url='http://www.beproud.jp/',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Plugins',
        'Framework :: Django',
        'Framework :: Django :: 1.8',
        'Framework :: Django :: 1.10',
        'Framework :: Django :: 1.11',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    include_package_data=True,
    packages=find_packages(),
    namespace_packages=['beproud', 'beproud.django'],
    install_requires=[
        'Django>=1.8',
        'zenhan>=0.4',
        'six',
    ],
    test_suite='tests.main',
    zip_safe=False,
)



# vim:fileencoding=utf-8

import warnings

from django.http import JsonResponse

__all__ = (
    'JSONResponse',
)


class JSONResponse(JsonResponse):
    """
    HttpResponse descendant, which return response with ``application/json`` mimetype.
    """
    def __init__(self, data={}, status=200, content_type='application/json'):
        warnings.warn('beproud.django.commons.http.JSONResponse was deprecated. Use django.http.JsonResponse instead.', DeprecationWarning)
        super(JSONResponse, self).__init__(data, status=status, content_type=content_type)
